@@ -1,6 +1,7 @@
 rescale <-
   function(pc, pd, d.prime, std.err, 
-           method = c("duotrio", "threeAFC", "twoAFC", "triangle"))
+           method = c("duotrio", "tetrad", "threeAFC", "twoAFC",
+             "triangle")) 
 {
   m <- match.call(expand.dots = FALSE)
   m[[1]] <- as.name("list")
@@ -107,7 +108,8 @@ pd2pc <- function(pd, Pguess) {
 
 psyfun <-
   function(d.prime,
-           method = c("duotrio", "threeAFC", "twoAFC", "triangle"))
+           method = c("duotrio", "tetrad", "threeAFC", "twoAFC",
+             "triangle")) 
 ### Maps d.prime to pc for sensory discrimination protocols
   
 ### arg: d.prime: non-negative numeric vector
@@ -117,6 +119,7 @@ psyfun <-
   stopifnot(all(is.numeric(d.prime)) && all(d.prime >= 0))
   psyFun <- switch(method,
                    duotrio = duotrio()$linkinv,
+                   tetrad = tetrad()$linkinv,
                    triangle = triangle()$linkinv,
                    twoAFC = twoAFC()$linkinv,
                    threeAFC = threeAFC()$linkinv)
@@ -131,7 +134,8 @@ psyfun <-
 }
 
 psyinv <- function(pc, 
-           method = c("duotrio", "threeAFC", "twoAFC", "triangle"))
+           method = c("duotrio", "tetrad", "threeAFC", "twoAFC",
+             "triangle")) 
 ### Maps pc to d.prime for sensory discrimination protocols
 
 ### arg: pc: numeric vector; 0 <= pc <= 1
@@ -141,6 +145,7 @@ psyinv <- function(pc,
   stopifnot(all(is.numeric(pc)) && all(pc >= 0) && all(pc <= 1))
   psyInv <- switch(method,
                    duotrio = duotrio()$linkfun,
+                   tetrad = tetrad()$linkfun,
                    triangle = triangle()$linkfun,
                    twoAFC = twoAFC()$linkfun,
                    threeAFC = threeAFC()$linkfun)
@@ -156,7 +161,8 @@ psyinv <- function(pc,
 
 psyderiv <-
   function(d.prime, 
-           method = c("duotrio", "threeAFC", "twoAFC", "triangle"))
+           method = c("duotrio", "tetrad", "threeAFC", "twoAFC",
+             "triangle")) 
 ### Computes the derivative of the psychometric functions at some
 ### d.prime for sensory discrimination protocols.
   
@@ -167,6 +173,7 @@ psyderiv <-
   stopifnot(all(is.numeric(d.prime)) && all(d.prime >= 0))
   psyDeriv <- switch(method,
                      duotrio = duotrio()$mu.eta,
+                     tetrad = tetrad()$mu.eta,
                      triangle = triangle()$mu.eta,
                      twoAFC = twoAFC()$mu.eta,
                      threeAFC = threeAFC()$mu.eta)
@@ -208,7 +215,8 @@ psyderiv <-
 ##   }
 ## }
 
-test.crit <- function(xcr, sample.size, p.correct = 0.5, alpha = 0.05, test)
+test.crit <-
+  function(xcr, sample.size, p.correct = 0.5, alpha = 0.05, test)
 ### Is xcr the critical value of a one-tailed binomial test?
 ### Result: boolean
 
@@ -331,8 +339,17 @@ normalPvalue <-
   stopifnot(all(is.finite(statistic)))
   p.value <-
     switch(alternative,
-           "greater" = pnorm(statistic, lower = FALSE),
-           "less" = pnorm(statistic, lower = TRUE),
-           "two.sided" = 2 * pnorm(abs(statistic), lower = FALSE))
+           "greater" = pnorm(statistic, lower.tail = FALSE),
+           "less" = pnorm(statistic, lower.tail = TRUE),
+           "two.sided" = 2 * pnorm(abs(statistic), lower.tail = FALSE))
   return(p.value)
 }
+
+
+## Do not partially match arguments.
+## If possible give functions explicitly named  arguments - preferably
+##   with default values.
+## Value readability over speed.
+## Value accuracy over speed.
+## Use small functions with conceptual - easy-to-understand tasks.
+## 
